# -*- coding: utf-8 -*-
from django.views.defaults import page_not_found
from django.views.defaults import server_error
from django.views.defaults import bad_request
from django.views.defaults import permission_denied

__author__ = "Md Nazrul Islam<connect2nazrul@gmail.com>"

ERROR_404_TEMPLATE_NAME = '404.html'
ERROR_403_TEMPLATE_NAME = '403.html'
ERROR_400_TEMPLATE_NAME = '400.html'
ERROR_500_TEMPLATE_NAME = '500.html'
